@@ -1,32 +1,28 @@
 (** {2 Command line} *)
 
-let concurrency, verbose, _keep_generated_files, debug, secondary, force_byte_compilation =
+let concurrency, verbose, debug, secondary, force_byte_compilation, static =
   let anon s = raise (Arg.Bad (Printf.sprintf "don't know what to do with %s\n" s)) in
   let concurrency = ref None in
   let verbose = ref false in
-  let keep_generated_files = ref false in
   let prog = Filename.basename Sys.argv.(0) in
   let debug = ref false in
   let secondary = ref false in
   let force_byte_compilation = ref false in
+  let static = ref false in
   Arg.parse
     [ "-j", Int (fun n -> concurrency := Some n), "JOBS Concurrency"
     ; "--verbose", Set verbose, " Set the display mode"
-    ; "--keep-generated-files", Set keep_generated_files, " Keep generated files"
+    ; "--keep-generated-files", Unit ignore, " Keep generated files"
     ; "--debug", Set debug, " Enable various debugging options"
     ; "--secondary", Set secondary, " Use the secondary compiler installation"
     ; ( "--force-byte-compilation"
       , Set force_byte_compilation
       , " Force bytecode compilation even if ocamlopt is available" )
+    ; "--static", Set static, " Build a static binary"
     ]
     anon
     (Printf.sprintf "Usage: %s <options>\nOptions are:" prog);
-  ( !concurrency
-  , !verbose
-  , !keep_generated_files
-  , !debug
-  , !secondary
-  , !force_byte_compilation )
+  !concurrency, !verbose, !debug, !secondary, !force_byte_compilation, !static
 ;;
 
 (** {2 General configuration} *)
@@ -1129,6 +1125,7 @@ let build
       | ".ml" -> Some (Filename.remove_extension fn ^ compiled_ml_ext)
       | _ -> None)
   in
+  let static_flags = if static then [ "-ccopt"; "-static" ] else [] in
   write_args "compiled_ml_files" compiled_ml_files;
   Process.run
     ~cwd:build_dir
@@ -1138,6 +1135,7 @@ let build
        ; obj_files
        ; [ "-args"; "compiled_ml_files" ]
        ; link_flags
+       ; static_flags
        ; allow_unstable_sources
        ])
 ;;
@@ -1152,10 +1150,9 @@ let rec rm_rf fn =
 
 and clear dir = List.iter (readdir dir) ~f:rm_rf
 
-let get_flags ocaml_system flags =
-  match List.assoc_opt ocaml_system flags with
-  | None -> []
-  | Some flags -> flags
+let rec get_flags system = function
+  | (set, f) :: r -> if List.mem system ~set then f else get_flags system r
+  | [] -> []
 ;;
 
 (** {2 Bootstrap process} *)
