@@ -9,7 +9,11 @@ val documentation : t -> string option
 val maintainers : t -> string list option
 
 (** example package info (used for project initialization ) *)
-val example : t
+val example
+  :  authors:string list option
+  -> maintainers:string list option
+  -> license:string list option
+  -> t
 
 val empty : t
 val to_dyn : t Dyn.builder
