@@ -1,16 +1,24 @@
 module Host : sig
-  type kind =
-    | Github
-    | Bitbucket
-    | Gitlab
-    | Sourcehut
-
-  type t =
+  type user_repo =
     { user : string
     ; repo : string
-    ; kind : kind
     }
 
+  type gitlab_repo =
+    | User_repo of user_repo
+    | Org_repo of
+        { org : string
+        ; proj : string
+        ; repo : string
+        }
+
+  type t =
+    | Github of user_repo
+    | Bitbucket of user_repo
+    | Gitlab of gitlab_repo
+    | Sourcehut of user_repo
+    | Codeberg of user_repo
+
   val homepage : t -> string
   val bug_reports : t -> string
 end
