@@ -1,4 +1,5 @@
 open Import
+open Memo.O
 module Opam_package = Package
 module P = Ocaml.Variant
 module Ps = Ocaml.Variant.Set
@@ -76,7 +77,6 @@ module DB = struct
   let create ~paths ~(lib_config : Lib_config.t) =
     let stdlib_dir = lib_config.stdlib_dir in
     let ext_lib = lib_config.ext_lib in
-    let open Memo.O in
     let+ builtins =
       let version = lib_config.ocaml_version in
       Meta.builtins ~stdlib_dir ~version
@@ -121,8 +121,13 @@ let to_dune_library (t : Findlib.Package.t) ~dir_contents ~ext_lib ~external_loc
     let main_module_name : Lib_info.Main_module_name.t = This None in
     let enabled = Memo.return Lib_info.Enabled_status.Normal in
     let requires =
+      let exports = Lib_name.Set.of_list (Findlib.Package.exports t) in
       Findlib.Package.requires t
-      |> List.map ~f:(fun name -> Lib_dep.direct (add_loc name))
+      |> List.map ~f:(fun name ->
+        let lib_dep =
+          if Lib_name.Set.mem exports name then Lib_dep.re_export else Lib_dep.direct
+        in
+        lib_dep (add_loc name))
     in
     let ppx_runtime_deps = List.map ~f:add_loc (Findlib.Package.ppx_runtime_deps t) in
     let special_builtin_support : (Loc.t * Lib_info.Special_builtin_support.t) option =
@@ -136,6 +141,7 @@ let to_dune_library (t : Findlib.Package.t) ~dir_contents ~ext_lib ~external_loc
     let public_headers = Lib_info.File_deps.External [] in
     let plugins = Findlib.Package.plugins t in
     let jsoo_runtime = Findlib.Package.jsoo_runtime t in
+    let wasmoo_runtime = Findlib.Package.wasmoo_runtime t in
     let melange_runtime_deps = Lib_info.File_deps.External [] in
     let preprocess = Preprocess.Per_module.no_preprocessing () in
     let virtual_ = None in
@@ -232,6 +238,7 @@ let to_dune_library (t : Findlib.Package.t) ~dir_contents ~ext_lib ~external_loc
       ~native_archives:(Files native_archives)
       ~foreign_dll_files:[]
       ~jsoo_runtime
+      ~wasmoo_runtime
       ~preprocess
       ~enabled
       ~virtual_deps
@@ -478,7 +485,6 @@ type t = DB.t
 let create =
   Per_context.create_by_name ~name:"findlib" (fun context ->
     Memo.lazy_ (fun () ->
-      let open Memo.O in
       let* context = Context.DB.get context in
       let* paths = Context.findlib_paths context
       and* lib_config =
