@@ -2,9 +2,7 @@ open Import
 
 (** Tie the knot between [Dune_engine] and [Dune_rules]. *)
 val init
-  :  ?action_runner:(Dune_engine.Action_exec.input -> Dune_engine.Action_runner.t option)
-  -> ?action_runners:(unit -> Dune_engine.Action_runner.t list)
-  -> stats:Dune_stats.t option
+  :  stats:Dune_stats.t option
   -> sandboxing_preference:Sandbox_mode.t list
   -> cache_config:Dune_cache.Config.t
   -> cache_debug_flags:Dune_engine.Cache_debug_flags.t
@@ -12,8 +10,7 @@ val init
   -> unit
 
 type build_system =
-  { conf : Dune_load.conf
-  ; contexts : Context.t list
+  { contexts : Context.t list
   ; scontexts : Super_context.t Context_name.Map.t
   }
 
