@@ -107,7 +107,7 @@ let impl_only_modules_defined_in_this_lib ~sctx ~scope lib =
 
 let cmj_glob = Glob.of_string_exn Loc.none "*.cmj"
 
-let cmj_includes ~(requires_link : Lib.t list Resolve.t) ~scope =
+let cmj_includes ~(requires_link : Lib.t list Resolve.t) ~scope lib_config =
   let project = Scope.project scope in
   let deps_of_lib lib =
     let info = Lib.info lib in
@@ -122,7 +122,7 @@ let cmj_includes ~(requires_link : Lib.t list Resolve.t) ~scope =
   let+ requires_link = requires_link in
   let deps = List.map requires_link ~f:deps_of_lib |> Dep.Set.of_list in
   Command.Args.S
-    [ Lib_flags.L.melange_emission_include_flags ~project requires_link
+    [ Lib_flags.L.melange_emission_include_flags ~project requires_link lib_config
     ; Hidden_deps deps
     ]
 ;;
@@ -136,13 +136,12 @@ let compile_info ~scope (mel : Melange_stanzas.Emit.t) =
          ~instrumentation_backend:(Lib.DB.instrumentation_backend (Scope.libs scope)))
     >>| Preprocess.Per_module.pps
   in
-  let merlin_ident = Merlin_ident.for_melange ~target:mel.target in
   let libraries =
     match mel.emit_stdlib with
+    | false -> mel.libraries
     | true ->
       let builtin_melange_dep = Lib_dep.Direct (mel.loc, Lib_name.of_string "melange") in
       builtin_melange_dep :: mel.libraries
-    | false -> mel.libraries
   in
   Lib.DB.resolve_user_written_deps
     (Scope.libs scope)
@@ -152,7 +151,6 @@ let compile_info ~scope (mel : Melange_stanzas.Emit.t) =
     libraries
     ~pps
     ~dune_version
-    ~merlin_ident
 ;;
 
 let js_targets_of_modules modules ~module_systems ~output =
@@ -271,6 +269,7 @@ let setup_emit_cmj_rules
   =
   let* compile_info = compile_info ~scope mel in
   let ctx = Super_context.context sctx in
+  let merlin_ident = Merlin_ident.for_melange ~target:mel.target in
   let f () =
     let* modules, obj_dir =
       Dir_contents.ocaml dir_contents
@@ -317,7 +316,7 @@ let setup_emit_cmj_rules
         ~requires_link
         ~requires_compile:direct_requires
         ~preprocessing:pp
-        ~js_of_ocaml:None
+        ~js_of_ocaml:(Js_of_ocaml.Mode.Pair.make None)
         ~opaque:Inherit_from_settings
         ~melange_package_name:None
         ~package:mel.package
@@ -326,6 +325,7 @@ let setup_emit_cmj_rules
     in
     let* () = Module_compilation.build_all cctx in
     let* requires_compile = Compilation_context.requires_compile cctx in
+    let* requires_hidden = Compilation_context.requires_hidden cctx in
     let stdlib_dir = (Compilation_context.ocaml cctx).lib_config.stdlib_dir in
     let+ () =
       let emit_and_libs_deps =
@@ -354,20 +354,20 @@ let setup_emit_cmj_rules
     in
     ( cctx
     , Merlin.make
-        ~requires:requires_compile
+        ~requires_compile
+        ~requires_hidden
         ~stdlib_dir
         ~flags
         ~modules
-        ~source_dirs:Path.Source.Set.empty
         ~libname:None
         ~preprocess:(Preprocess.Per_module.without_instrumentation mel.preprocess)
         ~obj_dir
-        ~ident:(Lib.Compile.merlin_ident compile_info)
+        ~ident:merlin_ident
         ~dialects:(Dune_project.dialects (Scope.project scope))
         ~modes:`Melange_emit )
   in
   let* () = Buildable_rules.gen_select_rules sctx compile_info ~dir in
-  Buildable_rules.with_lib_deps ctx compile_info ~dir ~f
+  Buildable_rules.with_lib_deps ctx merlin_ident ~dir ~f
 ;;
 
 module Runtime_deps = struct
@@ -468,7 +468,13 @@ let setup_entries_js
   let loc = mel.loc in
   let module_systems = mel.module_systems in
   let* requires_link = Memo.Lazy.force requires_link in
-  let includes = cmj_includes ~requires_link ~scope in
+  let* includes =
+    let+ lib_config =
+      let+ ocaml = Super_context.context sctx |> Context.ocaml in
+      ocaml.lib_config
+    in
+    cmj_includes ~requires_link ~scope lib_config
+  in
   let output = `Private_library_or_emit target_dir in
   let obj_dir = Obj_dir.of_local local_obj_dir in
   let* () =
@@ -526,6 +532,10 @@ let setup_js_rules_libraries =
           | None -> acc
           | Some sub -> if Lib.equal sub lib then acc else sub :: acc)
     in
+    let* lib_config =
+      let+ ocaml = Super_context.context sctx |> Context.ocaml in
+      ocaml.lib_config
+    in
     Memo.parallel_iter requires_link ~f:(fun lib ->
       let lib_compile_info =
         Lib.Compile.for_lib
@@ -546,7 +556,7 @@ let setup_js_rules_libraries =
           Memo.Lazy.force (Lib.Compile.requires_link lib_compile_info)
           |> Resolve.Memo.map ~f:(with_vlib_implementations lib)
         in
-        cmj_includes ~requires_link ~scope
+        cmj_includes ~requires_link ~scope lib_config
       in
       let+ () =
         setup_runtime_assets_rules
@@ -586,7 +596,7 @@ let setup_js_rules_libraries =
                  path in `import` / `require`. *)
               lib :: requires_link
             in
-            cmj_includes ~requires_link ~scope
+            cmj_includes ~requires_link ~scope lib_config
           in
           parallel_build_source_modules
             ~sctx
@@ -755,7 +765,7 @@ let gen_emit_rules sctx ~dir ({ stanza_dir; stanza } as for_melange) =
          ~hints:[ Pp.text "Move both `melange.emit' stanzas to the same level." ])
 ;;
 
-module Gen_rules = Import.Build_config.Gen_rules
+module Gen_rules = Build_config.Gen_rules
 
 let setup_emit_js_rules sctx ~dir =
   under_melange_emit_target ~sctx ~dir
