@@ -1,6 +1,5 @@
 (** ocamldep management *)
 
-open! Dune_engine
 open Import
 
 module Modules_data : sig
@@ -14,20 +13,32 @@ module Modules_data : sig
     ; obj_dir : Path.Build.t Obj_dir.t
     ; sctx : Super_context.t
     ; vimpl : Vimpl.t option
-    ; modules : Modules.t
+    ; modules : Modules.With_vlib.t
     ; stdlib : Ocaml_stdlib.t option
+    ; sandbox : Sandbox_config.t
     }
 end
 
-val deps_of :
-     Modules_data.t
+val deps_of
+  :  Modules_data.t
   -> ml_kind:Ml_kind.t
   -> Module.t
-  -> Module.t list Action_builder.t Memo.Build.t
+  -> Module.t list Action_builder.t Memo.t
 
-val read_deps_of :
-     obj_dir:Path.Build.t Obj_dir.t
-  -> modules:Modules.t
+val read_deps_of
+  :  obj_dir:Path.Build.t Obj_dir.t
+  -> modules:Modules.With_vlib.t
+  -> ml_kind:Ml_kind.t
+  -> Module.t
+  -> Module.t list Action_builder.t
+
+(** [read_immediate_deps_of ~obj_dir ~modules ~ml_kind unit] returns the
+    immediate dependencies found in the modules of [modules] for the file with
+    kind [ml_kind] of the module [unit]. If there is no such file with kind
+    [ml_kind], then an empty list of dependencies is returned. *)
+val read_immediate_deps_of
+  :  obj_dir:Path.Build.t Obj_dir.t
+  -> modules:Modules.With_vlib.t
   -> ml_kind:Ml_kind.t
   -> Module.t
   -> Module.t list Action_builder.t
