@@ -1,5 +1,4 @@
 open Import
-open Dune_file
 open Memo.O
 
 (* CR-someday jeremiedimino: This should be a memoized function [Super_context.t
@@ -11,14 +10,23 @@ let mlds_by_package_def =
     ~input:(module Super_context.As_memo_key)
     (fun sctx ->
       let ctx = Super_context.context sctx in
-      let* dune_files = Context.name ctx |> Only_packages.filtered_stanzas in
-      Memo.parallel_map dune_files ~f:(fun dune_file ->
-        Memo.parallel_map dune_file.stanzas ~f:(function
-          | Documentation d ->
-            let dir = Path.Build.append_source (Context.build_dir ctx) dune_file.dir in
-            let* dc = Dir_contents.get sctx ~dir in
-            let+ mlds = Dir_contents.mlds dc d in
-            let name = Package.name d.package in
+      Context.name ctx
+      |> Dune_load.dune_files
+      >>= Memo.parallel_map ~f:(fun dune_file ->
+        Dune_file.stanzas dune_file
+        >>= Memo.parallel_map ~f:(fun stanza ->
+          match Stanza.repr stanza with
+          | Documentation.T stanza ->
+            let+ mlds =
+              (let dir =
+                 Path.Build.append_source
+                   (Context.build_dir ctx)
+                   (Dune_file.dir dune_file)
+               in
+               Dir_contents.get sctx ~dir)
+              >>= Dir_contents.mlds ~stanza
+            in
+            let name = Package.name stanza.package in
             Some (name, mlds)
           | _ -> Memo.return None)
         >>| List.filter_opt)
