@@ -77,4 +77,7 @@ val has_META_files : t -> bool
 (** Whether the compiler supports occurrences indexation *)
 val supports_bin_annot_occurrences : t -> bool
 
+(** Whether the compiler supports the -H flag *)
+val supports_hidden_includes : t -> bool
+
 val add_std_cxx_flag : t -> bool
